#!/usr/bin/python3

import sys
import os
import tkinter as tk
from tkinter import filedialog
from PIL import Image, ImageFont, ImageDraw
from tempfile import mktemp

IMAGE_SCALING = 0.68
RESAMPLE_METHOD = Image.BILINEAR
JPEG_QUALITY = 10
FONT_FN = "times.ttf"
DEMOTIVATOR_LINE = "bruh"

# Step 1: Read the image from CLI argument or file dialog
if len(sys.argv) > 1:
    img_path = sys.argv[1]
else:
    root = tk.Tk()
    root.withdraw()
    img_path = filedialog.askopenfilename()
if not img_path:
    print("No image selected.")
    sys.exit()

img = Image.open(img_path)

# Step 2: Resize the image to fit within a 1024x768 canvas while keeping proportions and margins
canvas_width, canvas_height = 1024, 768
canvas_ratio = canvas_width / canvas_height
img_ratio = img.width / img.height
if img_ratio > canvas_ratio:
    # image is wider than canvas, adjust width to canvas width and adjust height
    img = img.resize(
        (canvas_width, int(canvas_width / img_ratio)), resample=RESAMPLE_METHOD
    )
else:
    # image is taller than canvas, adjust height to canvas height and adjust width
    img = img.resize(
        (int(canvas_height * img_ratio), canvas_height), resample=RESAMPLE_METHOD
    )

# Step 3: Downgrade the image to JPEG level 10
temporary_file = mktemp(".jpg")
img.save(temporary_file, quality=JPEG_QUALITY)
img = Image.open(temporary_file)

# Step 4: Resize the image proportionally to 68% using bilinear filtering
resized_width = int(img.width * IMAGE_SCALING)
resized_height = int(img.height * IMAGE_SCALING)
img = img.resize((resized_width, resized_height), resample=RESAMPLE_METHOD)

# Step 5: Desaturate the image
img = img.convert("L")

# Step 6: Create the demotivator
# Step 6.1: Calculate the size of the demotivator
border_thickness = 2
margin = 20
text_height = 40
demotivator_width = img.width + 2 * border_thickness + 2 * margin
demotivator_height = img.height + 2 * border_thickness + margin + text_height

# Step 6.2: Fill the demotivator with black color
demotivator = Image.new("L", (canvas_width, canvas_height), 0)

# Step 6.3: Put the image in the center of the demotivator
img_x = int((canvas_width - img.width) / 2)
img_y = int((canvas_height - demotivator_height) / 2)
demotivator.paste(img, (img_x, img_y))

# Step 6.4: Draw a white border around the image
border_rect = (
    img_x - border_thickness,
    img_y - border_thickness,
    img_x + img.width + border_thickness,
    img_y + img.height + border_thickness,
)
ImageDraw.Draw(demotivator).rectangle(border_rect, outline=255, width=border_thickness)

# Step 6.5: Add the text under the image
font = ImageFont.truetype(FONT_FN, 58)
text_x = img_x + margin
text_y = img_y + img.height + margin + border_thickness + 16
text_width, _ = ImageDraw.Draw(demotivator).textbbox(
    (text_x, text_y), DEMOTIVATOR_LINE, font=font
)[2:]
text_x += int((img.width - text_width) / 2)
ImageDraw.Draw(demotivator).text(
    (text_x, text_y), DEMOTIVATOR_LINE, font=font, fill=255
)

# Step 7: Save the demotivator to disk
output_path = os.path.join(os.getcwd(), "generated_demotivator.png")
demotivator.save(output_path)

# Step 8: Finish
if os.path.exists(temporary_file):
    os.remove(temporary_file)

if os.path.exists(output_path):
    print("Demotivator saved to", output_path)
else:
    print("Failed!")
