#!/usr/bin/python3
# Ported to Python 3 and optimized and personalized

import argparse
import fnmatch
import os
import sys

#  Recursively generate index.html files for
#  all subdirectories in a directory tree

index_file_name = "index.html"

CSS = """\
<style type="text/css">
body { background-color: #000000; font-family: monospace; font-size: 12pt; color: #40FF40; margin: 8px; padding: 0; text-align: left; }
ul { padding: 0; margin: 0; }
li { line-height: 14pt; list-style: none; list-style-type: none; padding: 3px 10px; margin: 3px 15px; display: block; }
.x { min-width: 600px; max-width: 1000px; border: 0; padding: 0; margin: 0 0 24pt 0; display: block; }
.i { background-color: #000040; color: #FFF; min-width: 600px; max-width: 1000px; border: 0; padding: 8px; margin: 0 0 24pt 0; display: block; }
.s { color: #404040; }
h1 { padding: 10px; margin: 15px; font-size: 12pt; }
a, a:link, a:visited { text-decoration: none; color: #40FF40; }
a:hover, a:focus, a:active { color: #408040; }
</style>"""


def process_dir(top_dir, opts):
    for parentdir, dirs, files in os.walk(top_dir):
        if not opts.dryrun:
            filename = index_file_name
            if opts.filename:
                filename = opts.filename
            indexjoin = lambda x: os.path.join(x, filename)
            if opts.index_here:
                writepath = indexjoin(os.path.dirname(os.path.realpath(__file__)))
            else:
                writepath = indexjoin(parentdir)
            index_file = open(writepath, "w", encoding="utf-8")
            index_file.write(
                """<!DOCTYPE html>
<html>
<head>
<title>{apd}</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
{css}
</head>
<body>
<center>
<div class="x">
<h1>{cd}</h1>
<li><a href="..">&lt;root directory&gt;</a></li>""".format(
                    apd=os.path.abspath(top_dir), css=CSS, cd=os.path.abspath(parentdir)
                )
            )
        for dirname in sorted(dirs):
            absolute_dir_path = os.path.join(parentdir, dirname)
            if not os.access(absolute_dir_path, os.W_OK):
                print(
                    "(!) The directory {} is not writable. Skipping.".format(
                        absolute_dir_path
                    )
                )
                continue
            if opts.verbose:
                print("Directory: {}".format(absolute_dir_path))
            if not opts.dryrun:
                caption = dirname.encode("utf-8", "xmlcharrefreplace").decode("utf-8")
                if opts.verbose:
                    print('- Sub-directory "{}"'.format(caption))
                index_file.write(
                    """
<li><a href="./{link}">&lt;{caption}&gt;</a></li>""".format(
                        link=dirname, caption=caption
                    )
                )
        for filename in sorted(files):
            if opts.filter and not fnmatch.fnmatch(filename, opts.filter):
                if opts.verbose:
                    print("Skipped: {}/{}".format(parentdir, filename))
                continue
            if opts.verbose:
                print('- File "{}"'.format(filename))
            filename_escaped = filename.encode("utf-8", "xmlcharrefreplace").decode(
                "utf-8"
            )
            if filename == index_file_name:
                continue
            try:
                size = int(os.path.getsize(os.path.join(parentdir, filename)))
                if not opts.dryrun:
                    index_file.write(
                        """
<li><a href="./{link}">{caption}</a>&nbsp;<span class="s">({size})</span></li>""".format(
                            link=filename,
                            caption=filename_escaped,
                            size=pretty_size(size),
                        )
                    )
            except Exception as e:
                print("(!) An error occured while writing file name:", e)
                print("    filename_utf8:")
                repr(filename)
                print("    filename_escaped:"),
                repr(filename_escaped)
        if not opts.dryrun:
            index_file.write(
                """
</div>
<div class="i">
Larry Holst
</div>
</center>
</body>
</html>
"""
            )
            index_file.close()


# bytes pretty-printing
UNITS_MAPPING = [
    (1024**5, " PiB"),
    (1024**4, " TiB"),
    (1024**3, " GiB"),
    (1024**2, " MiB"),
    (1024, " KiB"),
    (1, (" byte", " bytes")),
]


def pretty_size(bytes, units=UNITS_MAPPING):
    """
    Human-readable file sizes.

    https://pypi.python.org/pypi/hurry.filesize/
    """
    for factor, suffix in units:
        if bytes >= factor:
            break
    amount = int(bytes / factor)

    if isinstance(suffix, tuple):
        singular, multiple = suffix
        if amount == 1:
            suffix = singular
        else:
            suffix = multiple
    return str(amount) + suffix


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="""DESCRIPTION:
Generate directory index files recursively.
Start from current dir or from folder passed as first positional argument.
Optionally filter by file types with --filter "*.py"."""
    )
    parser.add_argument(
        "top_dir",
        nargs="?",
        action="store",
        help="top folder from which to start generating indexes, "
        "use current folder if not specified",
        default=os.getcwd(),
    )
    parser.add_argument(
        "--filter", "-f", help="only include files matching glob", required=False
    )
    parser.add_argument(
        "--verbose",
        "-v",
        action="store_true",
        help="***WARNING: this can take a very long time with complex file tree structures***"
        " verbosely list every processed file",
        required=False,
    )
    parser.add_argument(
        "--dryrun",
        "-d",
        action="store_true",
        help="don't write any files, just simulate the traversal",
        required=False,
    )
    parser.add_argument(
        "--index-here",
        "-I",
        action="store_true",
        help="write index.html to current directory",
        required=False,
    )
    parser.add_argument(
        "--filename", "-n", action="store", help="index.html file name", required=False
    )
    config = parser.parse_args(sys.argv[1:])
    process_dir(config.top_dir, config)
