#!/usr/bin/python3

# Licensed under Public Domain. No copyright for the code.

LICENSE_TEXT = """\
Licensed under Public Domain. No copyright for the code.
"""[
    :-1
]

from PIL import Image
from os import makedirs
import argparse
import os.path

# constants you can easily tweak
ICON_SIZES = (
    "16x16",
    "24x24",
    "32x32",
    "64x64",
    "128x128",
    "256x256",
)  # maximum is 256x256
COLOR_FORMATS = (
    "L",
    "P",
    "RGB",
    "RGBA",
)
FILENAME_FORMAT = "icon_{width}x{height}_{bit_depth}bit_{color_or_grayscale_label}"
# ------------------------------

overwrite_flag = False


is_format_grayscale = lambda color_format: color_format in ["L", "LA", "P"]
color_format_bit_depth = lambda color_format: {
    "L": 8,
    "LA": 16,
    "P": 8,  # can range in 1..8
    "RGB": 24,
    "RGBA": 32,
}.get(color_format, None)


def create_icon(png_path, output_dir, icon_sizes, color_formats):
    icons_left = len(icon_sizes) * len(color_formats)
    can_write = overwrite_flag
    ask_user_again = not overwrite_flag

    makedirs(output_dir, exist_ok=True)
    with Image.open(png_path) as img:
        for raw_sizes in icon_sizes:
            for color_format in color_formats:
                sizes = tuple([int(y) for y in raw_sizes.split("x")])
                icon = img.resize(sizes, Image.Resampling.LANCZOS).convert(color_format)

                is_grayscale = is_format_grayscale(color_format)
                base_fn = (
                    FILENAME_FORMAT.format(
                        width=sizes[0],
                        height=sizes[1],
                        color_format=color_format,
                        bit_depth=color_format_bit_depth(color_format),
                        color_or_grayscale_label="grayscale"
                        if is_grayscale
                        else "color",
                    )
                    + ".ico"
                )

                output_fp = os.path.join(output_dir, base_fn).replace("\\", "/")

                fp_exists = os.path.exists(output_fp)
                if ask_user_again:
                    try:
                        res = input(
                            "File %s exists. Overwrite? (yes/no/all/None): " % (base_fn)
                        ).strip()
                    except KeyboardInterrupt or EOFError:
                        raise SystemExit
                    if res:
                        res = res[0]
                    else:
                        raise SystemExit

                    if res == "y" or res == "a":
                        can_write = True
                    elif res.lower() == "n":
                        can_write = False
                    if res == "a" or res == "N":
                        ask_user_again = False

                can_write = not fp_exists or fp_exists and can_write

                if can_write:
                    icon.save(
                        output_fp, sizes=(sizes,), bitmap_format=True, format="ICO"
                    )
                    print("[+]", base_fn)
                    icons_left -= 1

    return icons_left


if __name__ == "__main__":
    from sys import argv, exit

    parser = argparse.ArgumentParser(
        description="Icon generator using Pillow. If you need to edit "
        + "something beyond these arguments, such as file name format, feel "
        + "free to open the file in your favorite text editor and navigate to "
        + "the 7th line.",
        epilog=LICENSE_TEXT,
    )
    parser.add_argument("input_file", help="the input image file to process")
    parser.add_argument(
        "output_dir",
        nargs="?",
        default=".",
        help="the output directory (default: current directory)",
    )
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="force overwrite of existing output files",
    )
    args = parser.parse_args()

    input_file = args.input_file.replace("\\", "/")
    output_dir = args.output_dir.replace("\\", "/")
    overwrite_flag = args.force

    print("Input file:", input_file)
    print("Output directory:", output_dir)
    print("Overwriting flag: %sset" % ("" if overwrite_flag else "not "))
    print("Format:", FILENAME_FORMAT + ".ico")
    print()

    res = create_icon(args.input_file, args.output_dir, ICON_SIZES, COLOR_FORMATS)
    print()

    if res != 0:
        print("Warning: Not all files were written")
        exit(1)

    print("Success!")
    exit(0)
